@@ -1,11 +1,11 @@
-"""Provides convenient access to an SQLObject or SQLAlchemy
-managed database."""
+"""Convenient access to an SQLObject or SQLAlchemy managed database."""
 
 import sys
 import time
 import logging
 
 import cherrypy
+from cherrypy import request
 from cherrypy.filters.basefilter import BaseFilter
 
 try:
@@ -57,6 +57,8 @@ if sqlalchemy:
     metadata = sqlalchemy.MetaData()
     try:
         from sqlalchemy.orm import scoped_session
+        # Create session with autoflush=False
+        # and autocommit=True (transactional=False)
         session = scoped_session(create_session)
         mapper = session.mapper # use session-aware mapper
     except ImportError: # SQLAlchemy < 0.4
@@ -113,7 +115,7 @@ bind_meta_data = bind_metadata = get_engine # alias names
 
 try:
     set
-except NameError:
+except NameError: # Python 2.3
     from sets import Set as set
 
 hub_registry = set()
@@ -149,7 +151,7 @@ if sqlobject:
             ConnectionHub.__init__(self)
 
         def _is_interesting_version(self):
-            "Return True only if version of MySQLdb <= 1.0."
+            """Return True only if version of MySQLdb <= 1.0."""
             import MySQLdb
             module_version = MySQLdb.version_info[0:2]
             major = module_version[0]
@@ -200,7 +202,7 @@ if sqlobject:
             self.threadingLocal = threading_local()
 
         def begin(self, conn=None):
-            "Starts a transaction."
+            """Start a transaction."""
             if not self.supports_transactions:
                 return conn
             if not conn:
@@ -215,7 +217,7 @@ if sqlobject:
             return trans
 
         def commit(self):
-            "Commits the current transaction."
+            """Commit the current transaction."""
             if not self.supports_transactions:
                 return
             try:
@@ -226,7 +228,7 @@ if sqlobject:
                 self.threadingLocal.connection.commit()
 
         def rollback(self):
-            "Rolls back the current transaction."
+            """Rollback the current transaction."""
             if not self.supports_transactions:
                 return
             try:
@@ -237,7 +239,7 @@ if sqlobject:
                 self.threadingLocal.connection.rollback()
 
         def end(self):
-            "Ends the transaction, returning to a standard connection."
+            """End the transaction, returning to a standard connection."""
             if not self.supports_transactions:
                 return
             try:
@@ -322,17 +324,17 @@ def set_db_uri(dburi, package=None):
         })
 
 def commit_all():
-    "Commits the Transactions in all registered hubs (for this thread)"
+    """Commits the transactions in all registered hubs (for this thread)."""
     for hub in hub_registry:
         hub.commit()
 
 def rollback_all():
-    "Rolls back the Transactions in all registered hubs (for this thread)"
+    """Rollback the transactions in all registered hubs (for this thread)."""
     for hub in hub_registry:
         hub.rollback()
 
 def end_all():
-    "Ends the Transactions in all registered hubs (for this thread)"
+    """End the transactions in all registered hubs (for this thread)."""
     for hub in hub_registry:
         hub.end()
 
@@ -348,7 +350,8 @@ def _use_sa(args=None):
     # check to see if sqlalchemy has been imported and configured
     return _engine is not None
 
-[run_with_transaction.when("not _use_sa(args)")] # include "args" to avoid call being pre-cached
+# include "args" to avoid call being pre-cached
+[run_with_transaction.when("not _use_sa(args)")]
 def so_rwt(func, *args, **kw):
     log.debug("Starting SQLObject transaction")
     try:
@@ -363,14 +366,15 @@ def so_rwt(func, *args, **kw):
             commit_all()
             raise
         except:
-            # No need to "rollback" the sqlalchemy unit of work, because nothing
-            # has hit the db yet.
+            # No need to "rollback" the sqlalchemy unit of work,
+            # because nothing has hit the db yet.
             rollback_all()
             raise
     finally:
         end_all()
 
-[restart_transaction.when("not _use_sa(args)")] # include "args" to avoid call being pre-cached
+# include "args" to avoid call being pre-cached
+[restart_transaction.when("not _use_sa(args)")]
 def so_restart_transaction(args):
     #log.debug("ReStarting SQLObject transaction")
     # Disable for now for compatibility
@@ -387,10 +391,8 @@ def dispatch_exception(exception, args, kw):
     try:
         output = dispatch_error(
             controller, real_func, None, exception, *args, **kw)
-
     except dispatch.NoApplicableMethods:
         raise exc_type, exc_value, exc_trace
-
     else:
         del exc_trace
         return output
@@ -398,81 +400,102 @@ def dispatch_exception(exception, args, kw):
 # include "args" to avoid call being pre-cached
 [run_with_transaction.when("_use_sa(args)")]
 def sa_rwt(func, *args, **kw):
-    log.debug("New SA transaction")
-    req = cherrypy.request
-    req.sa_transaction = make_sa_transaction(session)
-
+    log.debug("Starting SA transaction")
+    request.sa_transaction = make_sa_transaction(session)
     try:
-        retval = func(*args, **kw)
-
-    except (cherrypy.HTTPRedirect, cherrypy.InternalRedirect):
-        log.debug('this is only a redirect')
-        # If a redirect happens; commit and proceed with redirect
-        if sa_tr_active(req.sa_transaction):
-            req.sa_transaction.commit()
-        raise
-
-    except:
-        log.debug('this is an exception, ROLLBACK now!')
-        # If any other exception happens; rollback and re-raise error
-        if sa_tr_active(req.sa_transaction):
-            req.sa_transaction.rollback()
-        raise
-
-    # If the call was successful; commit and proceed
-    if sa_tr_active(req.sa_transaction):
-        log.debug('The transaction was successful, COMMIT now!')
-        req.sa_transaction.commit()
-
+        try:
+            retval = func(*args, **kw)
+        except (cherrypy.HTTPRedirect, cherrypy.InternalRedirect):
+            # If a redirect happens, commit and proceed with redirect
+            if sa_transaction_active():
+                log.debug('Redirect in active transaction - will commit now')
+                if hasattr(session, 'commit'):
+                    session.commit()
+                else: # SA < 0.4
+                    request.sa_transaction.commit()
+            else:
+                log.debug('Redirect in inactive transaction')
+            raise
+        except:
+            # If any other exception happens, rollback and re-raise error
+            if sa_transaction_active():
+                log.debug('Error in active transaction - will rollback now')
+                if hasattr(session, 'rollback'):
+                    session.rollback()
+                else: # SA < 0.4
+                    request.sa_transaction.rollback()
+            else:
+                log.debug('Error in inactive transaction')
+            raise
+        # If the call was successful, commit and proceed
+        if sa_transaction_active():
+            log.debug('Transaction is still active - will commit now')
+            try:
+                session.commit()
+            except AttributeError: # SA < 0.4
+                request.sa_transaction.commit()
+        else:
+            log.debug('Transaction is already inactive')
+    finally:
+        log.debug('Ending SA transaction')
+        session.close()
     return retval
 
 # include "args" to avoid call being pre-cached
 [restart_transaction.when("_use_sa(args)")]
 def sa_restart_transaction(args):
     log.debug("Restarting SA transaction")
-    req = cherrypy.request
-    if sa_tr_active(req.sa_transaction):
-        req.sa_transaction.rollback()
-    session.clear()
-    req.sa_transaction = make_sa_transaction(session)
-
-def sa_tr_active(tr):
-    if hasattr(session, 'context'):
-        # SA 0.3
-        return tr.session.transaction
+    if sa_transaction_active():
+        log.debug('Transaction is still active - will rollback now')
+        try:
+            session.rollback()
+        except AttributeError: # SA < 0.4
+            request.sa_transaction.rollback()
     else:
-        # SA 0.4 (effectively always active - commit or rollback don't fail)
-        return True
+        log.debug('Transaction is already inactive')
+    session.close()
+    request.sa_transaction = make_sa_transaction(session)
 
 def make_sa_transaction(session):
-    if hasattr(session, 'context'):
-        # SA 0.3
-        tr = session.create_transaction()
-        return tr
+    """Create a new transaction in an SA session."""
+    try:
+        return session.begin()
+    except AttributeError: # SA < 0.4
+        return session.create_transaction()
 
-    else:
-        # SA 0.4
-        session.begin()
-        return session
+def sa_transaction_active():
+    """Check whether SA transaction is still active."""
+    try:
+        return session().is_active
+    except (TypeError, AttributeError): # SA < 0.4.7
+        try:
+            transaction = request.sa_transaction
+        except AttributeError:
+            return False
+        try:
+            return transaction and transaction.is_active
+        except AttributeError: # SA < 0.4.3
+            return transaction.session.transaction
 
 def so_to_dict(sqlobj):
-    "Converts SQLObject to a dictionary based on columns"
+    """Convert SQLObject to a dictionary based on columns."""
     d = {}
-    if sqlobj == None:
-        # stops recursion
-        return d
+    if sqlobj is None:
+        return d # stops recursion
     for name in sqlobj.sqlmeta.columns.keys():
         d[name] = getattr(sqlobj, name)
-    "id must be added explicitly"
-    d["id"] = sqlobj.id
+    d['id'] = sqlobj.id # id must be added explicitly
     if sqlobj._inheritable:
         d.update(so_to_dict(sqlobj._parent))
         d.pop('childName')
     return d
 
 def so_columns(sqlclass, columns=None):
-    """Returns a dict with all columns from a SQLObject including those from
-    InheritableSO's bases"""
+    """Return a dict with all columns from a SQLObject.
+
+    This includes the columns from InheritableSO's bases.
+
+    """
     if columns is None:
         columns = {}
     columns.update(filter(lambda i: i[0] != 'childName',
@@ -482,8 +505,11 @@ def so_columns(sqlclass, columns=None):
     return columns
 
 def so_joins(sqlclass, joins=None):
-    """Returns a list with all joins from a SQLObject including those from
-    InheritableSO's bases"""
+    """Return a list with all joins from a SQLObject.
+
+    The list includes the columns from InheritableSO's bases.
+
+    """
     if joins is None:
         joins = []
     joins.extend(sqlclass.sqlmeta.joins)
