@@ -136,9 +136,9 @@ def lazify(func):
         return lazystr
     return newfunc
 
+[jsonify.when("isinstance(obj, lazystring)")]
 def jsonify_lazystring(obj):
     return unicode(obj)
-jsonify.when("isinstance(obj, lazystring)")(jsonify_lazystring)
 
 lazy_gettext = lazify(plain_gettext)
 lazy_ngettext = lazify(plain_ngettext)
