@@ -1,13 +1,9 @@
-'''
-the visit and identity management *plugins* are defined here
-'''
+"""The visit and identity management *plugins* are defined here."""
 
-import cherrypy
-import sha
-import datetime
-import time
 import base64
 
+from cherrypy import request
+
 try:
     from sqlobject import *
 except ImportError:
@@ -29,11 +25,11 @@ log = logging.getLogger("turbogears.identity")
 # Interface for the TurboGears extension
 def start_extension():
     # Bail out if the application hasn't enabled this extension
-    if not turbogears.config.get("identity.on", False):
+    if not turbogears.config.get('identity.on', False):
         return
 
     # Identity requires that Visit tracking be enabled
-    if not turbogears.config.get("visit.on", False):
+    if not turbogears.config.get('visit.on', False):
         raise IdentityConfigurationException(
                 "Visit tracking must be enabled (visit.on)")
 
@@ -46,11 +42,9 @@ def start_extension():
 
 def shutdown_extension():
     # Bail out if the application hasn't enabled this extension
-    if not turbogears.config.get("identity.on", False):
+    if not turbogears.config.get('identity.on', False):
         return
-
     log.info("Identity shutting down")
-    pass
 
 
 def create_extension_model():
@@ -68,28 +62,30 @@ class IdentityVisitPlugin(object):
         # When retrieving identity information from the form, use the following
         # form field names. These fields will be removed from the post data to
         # prevent the controller from receiving unexpected fields.
-        self.user_name_field = get("identity.form.user_name", "user_name")
-        self.password_field = get("identity.form.password", "password")
-        self.submit_button_name = get("identity.form.submit", "login")
+        self.user_name_field = get('identity.form.user_name', 'user_name')
+        self.password_field = get('identity.form.password', 'password')
+        self.submit_button_name = get('identity.form.submit', 'login')
 
         # Sources for identity information and the order in which they should be
         # checked. These terms are mapped to methods by prepending
         # "identity_from_".
-        sources = get("identity.source", "form,http_auth,visit").split(",")
+        sources = get('identity.source', 'form,http_auth,visit').split(',')
         self.identity_sources = list()
         for s in sources:
             try:
-                source_method = getattr(self, "identity_from_" + s)
+                source_method = getattr(self, 'identity_from_' + s)
             except AttributeError:
-                raise IdentityConfigurationException("Invalid identity source: %s" % s)
+                raise IdentityConfigurationException(
+                    "Invalid identity source: %s" % s)
             self.identity_sources.append(source_method)
 
     def identity_from_request(self, visit_key):
-        '''
-        Retrieve identity information from the HTTP request. Checks first for
-        form fields defining the identity then for a cookie. If no identity
-        is found, returns an anonymous identity.
-        '''
+        """Retrieve identity information from the HTTP request.
+
+        Checks first for form fields defining the identity then for a cookie.
+        If no identity is found, returns an anonymous identity.
+
+        """
         identity = None
         log.debug("Retrieving identity for visit: %s", visit_key)
         for source in self.identity_sources:
@@ -103,24 +99,23 @@ class IdentityVisitPlugin(object):
         return identity
 
     def decode_basic_credentials(self, credentials):
-        '''
-        Decode base64 user_name:password credentials used in Basic Auth. Returned
-        with username in element 0 and password in element 1.
-        '''
-        return base64.decodestring(credentials.strip()).split(":")
+        """Decode base64 user_name:password credentials used in Basic Auth.
+
+        Returned with username in element 0 and password in element 1.
+
+        """
+        return base64.decodestring(credentials.strip()).split(':')
 
     def identity_from_http_auth(self, visit_key):
-        '''
-        Only basic auth is handled at the moment.
-        '''
+        """Only basic auth is handled at the moment."""
         try:
-            authorisation = cherrypy.request.headers['Authorization']
+            authorisation = request.headers['Authorization']
         except KeyError:
             return None
 
-        authScheme, schemeData = authorisation.split(" ", 1)
+        authScheme, schemeData = authorisation.split(' ', 1)
         # Only basic is handled at the moment
-        if "basic" != authScheme.lower():
+        if authScheme.lower() != 'basic':
             log.error("HTTP Auth is not basic")
             return None
 
@@ -133,14 +128,15 @@ class IdentityVisitPlugin(object):
         return self.provider.load_identity(visit_key)
 
     def identity_from_form(self, visit_key):
-        '''
-        Inspect the form to pull out identity information. Must have fields for
-        user name, password, and a login submit button.
+        """Inspect the form to pull out identity information.
 
-        Returns an identity dictionary or none if the form contained no identity
-        information or the information was incorrect.
-        '''
-        params = cherrypy.request.params
+        Must have fields for user name, password, and a login submit button.
+
+        Returns an identity dictionary or none if the form contained no
+        identity information or the information was incorrect.
+
+        """
+        params = request.params
         # only try to process credentials for login forms
         if params.has_key(self.submit_button_name):
             try:
@@ -148,9 +144,9 @@ class IdentityVisitPlugin(object):
                 user_name = params.pop(self.user_name_field)
                 pw = params.pop(self.password_field)
                 # just lose the submit button to prevent passing to final controller
-                submit= params.pop(self.submit_button_name, None)
-                submit_x = params.pop("%s.x" % self.submit_button_name, None)
-                submit_y = params.pop("%s.y" % self.submit_button_name, None)
+                submit = params.pop(self.submit_button_name, None)
+                submit_x = params.pop('%s.x' % self.submit_button_name, None)
+                submit_y = params.pop('%s.y' % self.submit_button_name, None)
                 set_login_attempted(True)
                 identity = self.provider.validate_identity(user_name, pw, visit_key)
                 if identity is None:
@@ -165,7 +161,7 @@ class IdentityVisitPlugin(object):
 
     def record_request(self, visit):
         # default to keeping the identity filter off
-        if not turbogears.config.get("identity.on", True):
+        if not turbogears.config.get('identity.on', True):
             log.debug("Identity is not enabled. Setting current identity to None")
             set_current_identity(None)
             return
